@@ -7,7 +7,10 @@ import re
 from weakref import ref
 import ctypes, struct, mmap
 # local
-import model # TODO check ctypes_tools.bytes2array in ptrace
+#from model import bytes2array # TODO check ctypes_tools.bytes2array in ptrace
+
+import logging
+log = logging.getLogger('memory_mapping')
 
 PROC_MAP_REGEX = re.compile(
     # Address range: '08048000-080b0000 '
@@ -73,9 +76,10 @@ class MemoryMapping:
         return text
     __repr__ = __str__
 
-    def search(self, bytestr):
-        process = self._process()
+    def __len__(self):
+      return int(self.end - self.start)
 
+    def search(self, bytestr):
         bytestr_len = len(bytestr)
         buf_len = 64 * 1024 
 
@@ -167,21 +171,30 @@ class MemoryMapping:
             address += chunk_length
         return ''.join(string), truncated
 
+    def __getstate__(self):
+      d = dict(self.__dict__)
+      del d['_process'] #= d['_process'].pid
+      #d['local_mmap'] = model.array2bytes(d['local_mmap'])
+      del d['local_mmap']
+      return d
+
+    #def __setstate__(self, state):
+    #  for k,v in state.items():
+    #    self.__dict__[k] = v
+    #  #d['_process'] = 
+    #  #d['local_mmap'] = model.bytes2array(d['local_mmap'],ctypes.c_ubyte)
+     
 
 class MemoryDumpMemoryMapping(MemoryMapping):
     """ A memoryMapping wrapper around a memory file dump"""
-    def __init__(self, memdump, start, end):
+    def __init__(self, memdump, start, end, permissions='rwx-', offset=0x0, major_device=0x0, minor_device=0x0, inode=0x0, pathname='MEMORYDUMP'):
+        MemoryMapping.__init__(self, self, start, end, permissions, offset, major_device, minor_device, inode, pathname)
         self._process = None
-        self.start = start
-        self.end = end
-        self.permissions = 'rwx-'
-        self.offset = 0x0
-        self.major_device = 0x0
-        self.minor_device = 0x0
-        self.inode = 0x0
-        self.pathname = 'MEMORYDUMP'
         self.local_mmap = mmap.mmap(memdump.fileno(), end-start, access=mmap.ACCESS_READ)
 
+    def _err(self):
+        raise 
+        
     def search(self, bytestr):
         self.local_mmap.find(bytestr)
 
@@ -197,10 +210,11 @@ class MemoryDumpMemoryMapping(MemoryMapping):
         return data
 
     def readStruct(self, address, structType):
+        from model import bytes2array # TODO check ctypes_tools.bytes2array in ptrace
         laddr = self.vtop(address)
         structLen = ctypes.sizeof(structType)
         st = self.local_mmap[laddr:laddr+structLen]
-        structtmp = model.bytes2array(st, ctypes.c_ubyte)
+        structtmp = bytes2array(st, ctypes.c_ubyte)
         struct = structType.from_buffer(structtmp)
         return struct
 
@@ -218,6 +232,99 @@ class MemoryDumpMemoryMapping(MemoryMapping):
         text += " (%s)" % self.permissions
         return text
 
+def fileMemoryMapping_process(self):
+  ''' fake it like a process and mmap it now'''
+  import model
+  if self.local_mmap is None:
+    if hasattr(self.memdump,'fileno'):
+      self.local_mmap = mmap.mmap(self.memdump.fileno(), self.end-self.start, access=mmap.ACCESS_READ)
+      log.info('Lazy Memory Mapping content mmap-ed() : %s'%(self))
+    else:
+      # use that or mmap, anyway, we need to convert to ctypes :/ that costly
+      self.local_mmap = model.bytes2array(self.memdump.read(), ctypes.c_ubyte)
+      log.info('Lazy Memory Mapping content loaded : %s'%(self))
+  return self
+
+def FileMemoryMapping(memoryMapping, memdump):
+  """ A memoryMapping wrapper backed by a around a memory file dump for mmap() 
+  @param memoryMapping: a MemoryMapping
+  @param memdump: memorydump File
+  """
+  import copy, types
+  p = None
+  m = None
+  # we del _process
+  if hasattr(memoryMapping,'_process'):
+    p = memoryMapping._process
+  # we keep local__map
+  if not hasattr(memoryMapping,'local_mmap'):
+    memoryMapping.local_mmap = None
+  memoryMapping._process = None
+  ret = copy.deepcopy(memoryMapping)
+  if hasattr(memoryMapping,'_process'):
+    memoryMapping._process = p
+  ret.memdump = memdump
+  ret._process = types.MethodType(fileMemoryMapping_process, ret, MemoryMapping)
+  #if memoryMapping.local_mmap is not None:
+  if not hasattr(ret,'local_mmap'):
+    ret.local_mmap = None
+  return ret
+
+def getFileBackedMemoryMapping(memoryMapping, memdump):
+  return FileBackedMemoryMapping(memdump, memoryMapping.start, memoryMapping.end, 
+              memoryMapping.permissions, memoryMapping.offset, memoryMapping.major_device, memoryMapping.minor_device, 
+              memoryMapping.inode, memoryMapping.pathname)
+
+class LazyMmap:
+  ''' lazy mmap no memory.
+   useless.
+  '''
+  def __init__(self,memdump):
+    memdump.seek(2**64)
+    self.size = memdump.tell()
+    self.memdump = memdump
+  
+  def __len__(self):
+    return self.size
+    
+  def __getitem__(self,key):
+    if type(key) == slice :
+      start = key.start
+      size = key.stop - key.start
+    elif type(key) == int :
+      start = key
+      size = 1
+    else :
+      raise ValueError('bad index type')
+    return self._get(start, size)
+  
+  def _get(self, offset,size):
+    import model 
+    self.memdump.seek(offset)
+    #me = mmap.mmap(memdump.fileno(), end-start, access=mmap.ACCESS_READ)
+    me = model.bytes2array(self.memdump.read(size) ,ctypes.c_ubyte)
+    return me
+
+class FileBackedMemoryMapping(MemoryDumpMemoryMapping):
+  '''
+    don't mmap the memoryMap. use the file to read offsets.
+  '''
+  def __init__(self, memdump, start, end, permissions='rwx-', offset=0x0, major_device=0x0, minor_device=0x0, inode=0x0, pathname='MEMORYDUMP'):
+    MemoryMapping.__init__(self, self, start, end, permissions, offset, major_device, minor_device, inode, pathname)
+    self.memdump = memdump
+    self.local_mmap = LazyMmap(self.memdump)
+    return    
+  def readWord(self, address):
+    """Address have to be aligned!"""
+    laddr = self.vtop(address)
+    size = ctypes.sizeof((ctypes.c_int))
+    word = ctypes.c_ulong.from_buffer_copy(self.local_mmap[laddr:laddr+size], 0).value # is non-aligned a pb ?
+    return word
+  def readArray(self, address, basetype, count):
+    laddr = self.vtop(address)
+    size = ctypes.sizeof((basetype *count))
+    array = (basetype *count).from_buffer_copy(self.local_mmap[laddr:laddr+size], 0)
+    return array
 
 def readProcessMappings(process):
     """
@@ -235,6 +342,7 @@ def readProcessMappings(process):
         mapsfile = openProc("%s/maps" % process.pid)
     except ProcError, err:
         raise ProcessError(process, "Unable to read process maps: %s" % err)
+    
     try:
         for line in mapsfile:
             line = line.rstrip()
