@@ -21,20 +21,47 @@ if ctypes.c_char_p.__name__ == 'c_char_p':
 if ctypes.Structure.__name__ == 'Structure':
   ctypes.original_Structure = ctypes.Structure
 
-__refs = list()
-__register = dict()
+#__refs = list()
+#__register = dict()
+
+class _book(object):
+  modules = set()
+  classes = dict()
+  refs = list()
+  def __init__(self):
+    pass
+  def addModule(self, mod):
+    self.modules.add(mod)
+  def addClass(self,cls):
+    self.classes[ctypes.POINTER(cls)] = cls
+  def addRef(self,obj):
+    self.refs.append(obj)
+  def getModules(self):
+    return set(self.modules)
+  def getClasses(self):
+    return dict(self.classes)
+  def isRegisteredType(self, typ):
+    return typ in self.classes.values()
+
+# central model book register
+__book = _book()
 
 def keepRef(obj):
   ''' Sometypes, your have to cast a c_void_p, You can keep ref in Ctypes object, 
     they might be transient (if obj == somepointer.contents).'''
-  __refs.append(obj)
+  #__refs.append(obj)
+  __book.addRef(obj)
   return
 
 def register(klass):
-  klass.classRef = __register
-  __register[ctypes.POINTER(klass)] = klass
+  #klass.classRef = __register
+  #__register[ctypes.POINTER(klass)] = klass
+  __book.addClass(klass)
+  klass.classRef = __book.classes
   return klass
 
+def registeredModules():
+  return sys.modules[__name__].__book.getModules()
 
 ''' returns if the address of the struct is in the mapping area
 '''
@@ -151,6 +178,10 @@ def pointer2bytes(attr,nbElement):
   array=(type(attr.contents)*nbElement).from_address(firstElementAddr)
   # we have an array type starting at attr.contents[0]
   return array2bytes(array)
+
+def isCTypes(obj):
+  ''' pointer or type'''
+  return  (type(obj).__module__ in ['ctypes','_ctypes']) 
     
 def isBasicType(obj):
   return  (type(obj).__module__ in ['ctypes','_ctypes','__builtin__']) 
@@ -345,9 +376,6 @@ class LoadableMembers(ctypes.Structure):
       return True
     # e) 
     if isPointerType(attr):
-      #### try to debug mem
-      setattr(self,attrname+'ContentAddress',getaddress(attr))
-      ####
       if attrname in self.expectedValues:
         # test if NULL is an option
         if not bool(attr):
@@ -469,6 +497,7 @@ class LoadableMembers(ctypes.Structure):
     if isCStringPointer(attr) : 
       # can't use basic c_char_p because we can't load in foreign memory
       attr_obj_address = getaddress(attr.ptr)
+      setattr(self,'__'+attrname,attr_obj_address)
       if not bool(attr_obj_address):
         log.debug('%s %s is a CString, the pointer is null (validation must have occurred earlier) '%(attrname, attr))
         return True
@@ -476,7 +505,6 @@ class LoadableMembers(ctypes.Structure):
       if not memoryMap :
         log.warning('Error on addr while fetching a CString. should not happen')
         return False
-      setattr(self,'__'+attrname,attr_obj_address)
       MAX_SIZE=255
       log.debug("%s %s is defined as a CString, loading from 0x%lx is_valid_address %s"%(
                       attrname,attr,attr_obj_address, is_valid_address(attr,mappings) ))
@@ -491,9 +519,6 @@ class LoadableMembers(ctypes.Structure):
       attr_obj_address=getaddress(attr)
       setattr(self,'__'+attrname,attr_obj_address)
       ####
-      previous=getattr(self,attrname+'ContentAddress')
-      if attr_obj_address !=previous:
-        log.warning('Change of pointer value between validation and loading... 0x%lx 0x%lx'%(previous,attr_obj_address))
       # memcpy and save objet ref + pointer in attr
       # we know the field is considered valid, so if it's not in memory_space, we can ignore it
       memoryMap = is_valid_address( attr, mappings, _attrType)
@@ -505,7 +530,7 @@ class LoadableMembers(ctypes.Structure):
       ##### VALID INSTR.
       attr.contents=_attrType.from_buffer_copy(memoryMap.readStruct(attr_obj_address, _attrType ))
       #####
-      log.debug("%s %s loaded memcopy from 0x%lx to 0x%lx"%(attrname, attr,attr_obj_address, (getaddress(attr))   ))
+      log.debug("%s %s loaded memcopy from 0x%lx to 0x%lx"%(attrname, attr, attr_obj_address, (getaddress(attr))   ))
       # recursive validation checks on new struct
       if not bool(attr):
         log.warning('Member %s is null after copy: %s'%(attrname,attr))
@@ -611,6 +636,8 @@ class LoadableMembers(ctypes.Structure):
       attr=getattr(self,field)
       member=self._attrToPyObject(attr,field,typ)
       setattr(my_self, field, member)
+    # save the original type (me) and the field
+    setattr(my_self, '_ctype_', type(self))
     return my_self
     
   def _attrToPyObject(self,attr,field,typ):
@@ -679,9 +706,15 @@ class pyObj(object):
       #print 'ELSE type: %s %s'%(type(attr), type(type(attr)) )
     return s
 
+  def __len__(self):
+    return self._len_
+    
   def findCtypes(self):
     ret = False
     for attrname,typ in self.__dict__.items():
+      # ignore _ctype_, it's a ctype class type, we know that.
+      if attrname == '_ctype_' :
+        continue
       attr = getattr(self, attrname)
       if self._attrFindCtypes(attr, attrname,typ ):
         log.warning('Found a ctypes in %s'%(attrname))
@@ -694,7 +727,7 @@ class pyObj(object):
         if self._attrFindCtypes(el, 'element', None):
           log.warning('Found a ctypes in array/tuple')
           return True
-    elif type(attr).__module__ == 'ctypes':
+    elif isCTypes(attr):
       log.warning('Found a ctypes in self  %s'%(attr))
       return True
     elif not hasattr(attr,'__dict__'):
@@ -704,6 +737,13 @@ class pyObj(object):
       ret = False
     return ret
 
+  def __iter__(self):
+    ''' iterate on a instance's type's _fields_ members following the original type field order '''
+    for k,typ in self._ctype_._fields_:
+      v = getattr(self,k)
+      yield (k,v,typ)
+    pass
+
 def findCtypesInPyObj(obj):
   ''' check function to help in unpickling errors correction '''
   ret = False
@@ -754,11 +794,17 @@ def createPOPOClasses( targetmodule ):
   _created=0
   for klass,typ in inspect.getmembers(targetmodule, inspect.isclass):
     if typ.__module__.startswith(targetmodule.__name__):
-      kpy = type('%s_py'%(klass),( pyObj ,),{})
+      kpy = type('%s.%s_py'%(targetmodule.__name__, klass),( pyObj ,),{})
+      # add the structure size to the class
+      if type(typ) == type(LoadableMembers) or type(typ) == type( ctypes.Union) :
+        setattr(kpy, '_len_',ctypes.sizeof(typ) )
+      else:
+        setattr(kpy, '_len_', None )
       # we have to keep a local (model) ref because the class is being created here.
       # and we have a targetmodule ref. because it's asked.
       # and another ref on the real module for the basic type, because, that is probably were it's gonna be used.
-      setattr(sys.modules[__name__], '%s_py'%(klass), kpy )
+      setattr(sys.modules[__name__], '%s.%s_py'%(targetmodule.__name__, klass), kpy )
+      #setattr(sys.modules[__name__], '%s_py'%(klass), kpy )
       setattr(targetmodule, '%s_py'%(klass), kpy )
       _created+=1
       if typ.__module__ != targetmodule.__name__: # copy also to generated
@@ -773,6 +819,9 @@ def registerModule( targetmodule ):
       in a lookup table
       Creates POPO's to be able to unpickle ctypes.
   '''
+  if targetmodule in registeredModules():
+    log.warning('Module %s already registered. Skipping.'%(targetmodule))
+    return
   _registered = 0
   for klass,typ in inspect.getmembers(targetmodule, inspect.isclass):
     if typ.__module__.startswith(targetmodule.__name__):
@@ -780,9 +829,15 @@ def registerModule( targetmodule ):
       _registered += 1
   # create POPO's
   createPOPOClasses( targetmodule )
+  __book.addModule(targetmodule)
   log.debug('registered %d types'%( _registered))
+  log.debug('regisered %d module total'%(len(__book.getModules())))
   return
 
+def isRegistered(cls):
+  #return cls in sys.modules[__name__].__dict__.values()
+  return __book.isRegisteredType(cls)
+
 # create local POPO ( lodableMembers )
 createPOPOClasses(sys.modules[__name__] )
 # register LoadableMembers 
